/**
 * Revealer - Adds a stylish hover pop-up with five animation effects for internal WordPress website links.
 * Exclusively on Envato Market: https://1.envato.market/revealer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

"use strict";

/** Document Ready. */
document.addEventListener( 'DOMContentLoaded', function() {
   
    /** Fastest way to detect external URLs. */
    let isExternal = function(url) {
        let domain = function(url) {
            return url.toLocaleLowerCase()
                      .replace( 'http://www.','' )
                      .replace( 'https://www.','' )
                      .replace( 'http://','' )
                      .replace( 'https://','' )
                      .split('/')[0];
        };

        return domain( location.href ) !== domain( url );
    };

    /** Detect local page anchors. */
    let isLocalAnchor = function( reference ) {

        return reference.pathname === window.location.pathname &&
            reference.protocol === window.location.protocol &&
            reference.host === window.location.host;
    };
    
    /** Popup Settings. */
    var settings = {
        target: mdp_revealer.selector,
        animation: mdp_revealer.animation,
        maxWidth: mdp_revealer.maxWidth,
        duration: [275, 250],
        allowHTML: true,
        delay: [300, 200],
        interactive: true,
        interactiveBorder: 15,
        theme: mdp_revealer.theme,
        flipOnUpdate: true
    };
    
    /** Arrow. */
    if ( mdp_revealer.arrow ) {
        settings['arrow'] = true;
        settings['arrowType'] = mdp_revealer.arrowType;
    }
    
    /** 
     * Invoked when the tippy begins to transition in. 
     * You can cancel showing by returning false from this lifecycle. 
     * Receives the instance as an argument. 
     **/
    settings['onShow'] = function ( instance ) {

        /** Do not process links with class .not-revealer */
        if ( instance.reference.classList.contains( 'not-revealer' ) ) { return false; }

        /** Target URL. */
        var targetURL = new URL( encodeURI( instance.reference.getAttribute( 'href' ) ), document.baseURI ).href;

        /** Process only internal URLs. */
        if ( isExternal( targetURL ) ) { return false; }

        /** Skip local Anchors. */
        if ( isLocalAnchor( instance.reference ) ) { return false; }
        
        /** Mark element to process only once. */
        if ( instance.reference.getAttribute( 'data-revealer' ) ) {
            tippy.hideAll({exclude: instance});
            return true;
        }
        instance.reference.setAttribute( 'data-revealer', '1' );
        
        /** Set up our HTTP request. */
        var xhr = new XMLHttpRequest();
        xhr.open( 'POST', mdp_revealer.ajaxurl, true );
        xhr.setRequestHeader( 'Content-Type', 'application/x-www-form-urlencoded;' );
        xhr.responseType = 'json';

        /** Setup our listener to process completed requests. */
        xhr.onload = function () {

            /** Process our return data. */
            if ( xhr.status >= 200 && xhr.status < 300 ) {
                if ( xhr.response.status ) {
                    instance.setContent( xhr.response.html );
                    instance.show();
                }
            } else {
                /** Request fails. */
                console.log('The request failed!');
            }
            
        };
        
        /** Connection error. */
        xhr.onerror = function() {
            console.log( 'Connection error.' );                    
        };

        /** Create and send a POST request. */
        xhr.send( 'action=get_page_by_url&url=' + targetURL );
        
        return false;
    };
    
    /** 
     * Lifecycle function invoked when the tippy has fully transitioned out and is unmounted from the DOM. 
     * Receives the instance as an argument.
     **/
    settings['onHidden'] = function ( instance ) {
        /** Remove Attribute to allow re-shown tooltip. */
        instance.reference.removeAttribute( 'data-revealer' );
    };

    tippy( 'html', settings );
   
} );
