<?php
/**
 * Revealer - Adds a stylish hover pop-up with five animation effects for internal WordPress website links.
 * Exclusively on Envato Market: https://1.envato.market/revealer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Revealer;

use Merkulove\Revealer;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement plugin settings.
 *
 * @since 1.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Settings {

	/**
	 * Revealer Plugin settings.
	 *
	 * @var array()
	 * @since 1.0.0
	 **/
	public $options = [];

	/**
	 * The one true Settings.
	 *
	 * @var Settings
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get plugin settings. */
		$this->get_options();

	}

	/**
	 * Render Tabs Headers.
	 *
	 * @param string $current - Selected tab key.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function render_tabs( $current = 'general' ) {

		/** Tabs array. */
		$tabs = [];
		$tabs['general'] = [
			'icon' => 'tune',
			'name' => esc_html__( 'General', 'revealer' )
		];

		$tabs['design'] = [
			'icon' => 'brush',
			'name' => esc_html__( 'Design', 'revealer' )
		];

        $tabs['assignments'] = [
            'icon' => 'flag',
            'name' => esc_html__( 'Assignments', 'revealer' )
        ];

		$tabs['css'] = [
			'icon' => 'code',
			'name' => esc_html__( 'Custom CSS', 'revealer' )
		];

		/** Activation tab enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			$tabs['activation'] = [
				'icon' => 'vpn_key',
				'name' => esc_html__( 'Activation', 'revealer' )
			];
		}

		$tabs['status'] = [
			'icon' => 'info',
			'name' => esc_html__( 'Status', 'revealer' )
		];

		$tabs['uninstall'] = [
			'icon' => 'delete_sweep',
			'name' => esc_html__( 'Uninstall', 'revealer' )
		];

		/** Render Tabs. */
		?>
        <aside class="mdc-drawer">
            <div class="mdc-drawer__content">
                <nav class="mdc-list">

                    <div class="mdc-drawer__header mdc-plugin-fixed">
                        <!--suppress HtmlUnknownAnchorTarget -->
                        <a class="mdc-list-item mdp-plugin-title" href="#wpwrap">
                            <i class="mdc-list-item__graphic" aria-hidden="true">
                                <img src="<?php echo esc_attr( Revealer::$url . 'images/logo-color.svg' ); ?>" alt="<?php echo esc_html__( 'Revealer', 'revealer' ) ?>">
                            </i>
                            <span class="mdc-list-item__text">
                                <?php echo esc_html__( 'Revealer', 'revealer' ) ?>
                                <sup><?php echo esc_html__( 'ver.', 'revealer' ) . esc_html( Revealer::$version ); ?></sup>
                            </span>
                        </a>
                        <button type="submit" name="submit" id="submit"
                                class="mdc-button mdc-button--dense mdc-button--raised">
                            <span class="mdc-button__label"><?php echo esc_html__( 'Save changes', 'revealer' ) ?></span>
                        </button>
                    </div>

                    <hr class="mdc-plugin-menu">
                    <hr class="mdc-list-divider">
                    <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Plugin settings', 'revealer' ) ?></h6>

					<?php

					// Plugin settings tabs
					foreach ( $tabs as $tab => $value ) {
						$class = ( $tab == $current ) ? ' mdc-list-item--activated' : '';
						echo "<a class='mdc-list-item " . $class . "' href='?post_type=revealer_record&page=mdp_revealer_settings&tab=" . $tab . "'><i class='material-icons mdc-list-item__graphic' aria-hidden='true'>" . $value['icon'] . "</i><span class='mdc-list-item__text'>" . $value['name'] . "</span></a>";
					}

					/** Helpful links. */
					$this->support_link();

					/** Activation Status. */
					PluginActivation::get_instance()->display_status();

					?>
                </nav>
            </div>
        </aside>
		<?php
	}

	/**
	 * Displays useful links for an activated and non-activated plugin.
	 *
	 * @since 1.0.0
     *
     * @return void
	 **/
	public function support_link() { ?>

        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Helpful links', 'revealer' ) ?></h6>

        <a class="mdc-list-item" href="https://docs.merkulov.design/tag/revealer/" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'collections_bookmark' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'Documentation', 'revealer' ) ?></span>
        </a>

		<?php if ( PluginActivation::get_instance()->is_activated() ) : /** Activated. */ ?>
            <a class="mdc-list-item" href="https://1.envato.market/revealer-support" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">mail</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Get help', 'revealer' ) ?></span>
            </a>
            <a class="mdc-list-item" href="https://1.envato.market/cc-downloads" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">thumb_up</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Rate this plugin', 'revealer' ) ?></span>
            </a>
		<?php endif; ?>

        <a class="mdc-list-item" href="https://1.envato.market/cc-merkulove" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'store' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'More plugins', 'revealer' ) ?></span>
        </a>
		<?php

	}

	/**
	 * Add plugin settings page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function add_settings_page() {

		add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );
		add_action( 'admin_init', [ $this, 'settings_init' ] );

	}

	/**
	 * Create General Tab.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
    public function general_tab() {

	    /** General Tab. */
	    $group_name = 'RevealerOptionsGroup';
	    $section_id = 'mdp_revealer_settings_page_general_section';
	    $option_name = 'mdp_revealer_settings';

	    /** Create settings section. */
	    register_setting( $group_name, $option_name );
	    add_settings_section( $section_id, '', null, $group_name );

	    /** Render Settings fields. */
	    add_settings_field( 'selector',             esc_html__( 'Selector:', 'revealer' ),              ['\Merkulove\Revealer\SettingsFields', 'selector'],             $group_name, $section_id );
	    add_settings_field( 'size',                 esc_html__( 'Pop-up Size:', 'revealer' ),           ['\Merkulove\Revealer\SettingsFields', 'size'],                 $group_name, $section_id );
	    add_settings_field( 'image_position',       esc_html__( 'Image position:', 'revealer' ),        ['\Merkulove\Revealer\SettingsFields', 'image_position'],       $group_name, $section_id );
	    add_settings_field( 'image_size',           esc_html__( 'Image size:', 'revealer' ),            ['\Merkulove\Revealer\SettingsFields', 'image_size'],           $group_name, $section_id );
	    add_settings_field( 'author_position',      esc_html__( 'Author position:', 'revealer' ),       ['\Merkulove\Revealer\SettingsFields', 'author_position'],      $group_name, $section_id );
	    add_settings_field( 'category_position',    esc_html__( 'Categories position:', 'revealer' ),   ['\Merkulove\Revealer\SettingsFields', 'category_position'],    $group_name, $section_id );
	    add_settings_field( 'comments_position',    esc_html__( 'Comments position:', 'revealer' ),     ['\Merkulove\Revealer\SettingsFields', 'comments_position'],    $group_name, $section_id );
	    add_settings_field( 'excerpt_length',       esc_html__( 'Excerpt Length:', 'revealer' ),        ['\Merkulove\Revealer\SettingsFields', 'excerpt_length'],       $group_name, $section_id );

    }

	/**
	 * Create Design Tab.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function design_tab() {

		/** General Tab. */
		$group_name = 'RevealerDesignOptionsGroup';
		$section_id = 'mdp_revealer_design_settings_page_general_section';
		$option_name = 'mdp_revealer_design_settings';

		/** Create settings section. */
		register_setting( $group_name, $option_name );
		add_settings_section( $section_id, '', null, $group_name );

		/** Render Settings fields. */
		add_settings_field( 'style',                esc_html__( 'Style:', 'revealer' ),                 ['\Merkulove\Revealer\SettingsFields', 'style'],                $group_name, $section_id );
		add_settings_field( 'arrow',                esc_html__( 'Arrow:', 'revealer' ),                 ['\Merkulove\Revealer\SettingsFields', 'arrow'],                $group_name, $section_id );
		add_settings_field( 'bgcolor',              esc_html__( 'Background Color:', 'revealer' ),      ['\Merkulove\Revealer\SettingsFields', 'bgcolor'],              $group_name, $section_id );
		add_settings_field( 'text_color',           esc_html__( 'Text Color:', 'revealer' ),            ['\Merkulove\Revealer\SettingsFields', 'text_color'],           $group_name, $section_id );
		add_settings_field( 'animation',            esc_html__( 'Animation:', 'revealer' ),             ['\Merkulove\Revealer\SettingsFields', 'animation'],            $group_name, $section_id );

	}

	/**
	 * Create Custom CSS Tab.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function custom_css_tab() {

		/** Custom CSS. */
		$group_name = 'RevealerCSSOptionsGroup';
		$section_id = 'mdp_revealer_settings_page_css_section';

		/** Create settings section. */
		register_setting( $group_name, 'mdp_revealer_css_settings' );
		add_settings_section( $section_id, '', null, $group_name );

    }

	/**
	 * Generate Settings Page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function settings_init() {

		/** General Tab. */
	    $this->general_tab();

		/** Design Tab. */
		$this->design_tab();

		/** Create Assignments Tab. */
		AssignmentsTab::get_instance()->add_settings();

		/** Create Custom CSS Tab. */
		$this->custom_css_tab();

		/** Activation Tab. */
		PluginActivation::get_instance()->add_settings();

		/** Create Status Tab. */
		StatusTab::get_instance()->add_settings();

		/** Create Uninstall Tab. */
		UninstallTab::get_instance()->add_settings();

	}

	/**
	 * Add admin menu for plugin settings.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function add_admin_menu() {

		add_menu_page(
			esc_html__( 'Revealer Settings', 'revealer' ),
			esc_html__( 'Revealer', 'revealer' ),
			'manage_options',
			'mdp_revealer_settings',
			[ $this, 'options_page' ],
			'data:image/svg+xml;base64,' . base64_encode( file_get_contents( Revealer::$path . 'images/logo-menu.svg' ) ),
			'58.8661'// Always change digits after "." for different plugins.
		);

	}

	/**
	 * Plugin Settings Page.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function options_page() {

		/** User rights check. */
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		} ?>
        <!--suppress HtmlUnknownTarget -->
        <form action='options.php' method='post'>
            <div class="wrap">

				<?php
				$tab = 'general';
				if ( isset ( $_GET['tab'] ) ) { $tab = $_GET['tab']; }

				/** Render "Revealer settings saved!" message. */
				SettingsFields::get_instance()->render_nags();

				/** Render Tabs Headers. */
				?><section class="mdp-aside"><?php $this->render_tabs( $tab ); ?></section><?php

				/** Render Tabs Body. */
				?><section class="mdp-tab-content mdp-tab-<?php echo esc_attr( $tab ) ?>"><?php

					/** General Tab. */
					if ( 'general' === $tab ) {
						echo '<h3>' . esc_html__( 'Revealer Settings', 'revealer' ) . '</h3>';
						settings_fields( 'RevealerOptionsGroup' );
						do_settings_sections( 'RevealerOptionsGroup' );

                    /** Design Tab. */
					} elseif ( 'design' === $tab ) {
						echo '<h3>' . esc_html__( 'Design Settings', 'revealer' ) . '</h3>';
						settings_fields( 'RevealerDesignOptionsGroup' );
						do_settings_sections( 'RevealerDesignOptionsGroup' );

                    /** Assignments Tab. */
					} elseif ( 'assignments' === $tab ) {
						echo '<h3>' . esc_html__( 'Assignments Settings', 'revealer' ) . '</h3>';
						settings_fields( 'RevealerAssignmentsOptionsGroup' );
						do_settings_sections( 'RevealerAssignmentsOptionsGroup' );
						AssignmentsTab::get_instance()->render_assignments();

                    /** Custom CSS Tab. */
					} elseif ( 'css' === $tab ) {
						echo '<h3>' . esc_html__( 'Custom CSS', 'revealer' ) . '</h3>';
						settings_fields( 'RevealerCSSOptionsGroup' );
						do_settings_sections( 'RevealerCSSOptionsGroup' );
						SettingsFields::get_instance()->custom_css();

						/** Activation Tab. */
					} elseif ( 'activation' === $tab ) {
						settings_fields( 'RevealerActivationOptionsGroup' );
						do_settings_sections( 'RevealerActivationOptionsGroup' );
						PluginActivation::get_instance()->render_pid();

                    /** Status tab. */
					} elseif ( 'status' === $tab ) {
						echo '<h3>' . esc_html__( 'System Requirements', 'revealer' ) . '</h3>';
						StatusTab::get_instance()->render_form();

					} /** Uninstall Tab. */
                    elseif ( 'uninstall' === $tab ) {
						echo '<h3>' . esc_html__( 'Uninstall Settings', 'revealer' ) . '</h3>';
						UninstallTab::get_instance()->render_form();
					}

					?>
                </section>
            </div>
        </form>

		<?php
	}

	/**
	 * Get plugin settings with default values.
	 *
	 * @since 1.0.0
	 * @access public
	 * @return void
	 **/
	public function get_options() {

		/** General Tab Options. */
		$options = get_option( 'mdp_revealer_settings' );

		/** Default values. */
		$defaults = [

			// General Tab
			'selector'          => 'article p a', // Selector.
			'size'              => '320', // Popup Size.
			'excerpt_length'    => '15', // Excerpt Length.
			'image_position'    => 'top', // Image Position.
			'image_size'        => 'medium', // Image size.
			'author_position'   => 'bottom', // Author Position.
			'category_position' => 'bottom', // Author Position.
			'comments_position' => 'without-comments', // Author Position.

            // Design Tab
			'style'             => 'revealer-rectangular', // Popup Style.
			'arrow'             => 'sharp', // Arrow.
			'bgcolor'           => '#fff', // Background color.
			'text_color'        => '#242424', // Text color.
			'animation'         => 'shift-toward', // Animation.

			// Custom CSS Tab
            'custom_css'        => '', // Custom CSS.
        ];

		$results = wp_parse_args( $options, $defaults );

        /** Design tab Options. */
		$mdp_revealer_design_settings = get_option( 'mdp_revealer_design_settings' );
		$results = wp_parse_args( $mdp_revealer_design_settings, $results );

		/** Custom CSS tab Options. */
		$mdp_revealer_css_settings = get_option( 'mdp_revealer_css_settings' );
		$results = wp_parse_args( $mdp_revealer_css_settings, $results );

		$this->options = $results;
	}

	/**
	 * Main Settings Instance.
	 *
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
	 * @return Settings
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Settings ) ) {
			self::$instance = new Settings;
		}

		return self::$instance;
	}

} // End Class Settings.
