<?php
/**
 * Revealer - Adds a stylish hover pop-up with five animation effects for internal WordPress website links.
 * Exclusively on Envato Market: https://1.envato.market/revealer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Revealer;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to render plugin settings fields.
 *
 * @since 2.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class SettingsFields {

	/**
	 * The one true SettingsFields.
	 *
	 * @var SettingsFields
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Render Selector field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function selector() {

		/** Render Selector input. */
		UI::get_instance()->render_input(
			Settings::get_instance()->options['selector'],
			esc_html__('Selector', 'revealer' ),
			esc_html__( 'Links by selector will be processed. You can specify multiple selectors separated by commas.', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[selector]',
				'id' => 'mdp_revealer_settings_selector'
			]
		);

	}

	/**
	 * Render Style field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function style() {

		/** Prepare options. */
		$options = [
			'revealer-rectangular'  => esc_html__( 'Rectangle', 'revealer' ),
            'revealer-rounded'      => esc_html__( 'Rounded rectangle', 'revealer' ),
            'revealer-background'   => esc_html__( 'Image in background', 'revealer' )
		];

		/** Render Style select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['style'],
			esc_html__('Style', 'revealer' ),
			esc_html__('Tooltip Outline Style', 'revealer' ),
			[
				'name' => 'mdp_revealer_design_settings[style]',
				'id' => 'mdp_revealer_design_settings_style'
			]
		);

	}

	/**
	 * Render Arrow field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function arrow() {

		/** Prepare options. */
		$options = [
			'without-arrow' => esc_html__( 'Without Arrow', 'revealer' ),
            'sharp'         => esc_html__( 'Sharp', 'revealer' ),
            'round'         => esc_html__( 'Round', 'revealer' )
		];

		/** Render Arrow select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['arrow'],
			esc_html__('Arrow', 'revealer' ),
			esc_html__('Arrow shape and outline type', 'revealer' ),
			[
				'name' => 'mdp_revealer_design_settings[arrow]',
				'id' => 'mdp_revealer_design_settings_arrow'
			]
		);

	}

	/**
	 * Render Animation field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function animation() {

		/** Prepare options. */
		$options = [
			'shift-away'    => esc_html__( 'Shift Away', 'revealer' ),
			'shift-toward'  => esc_html__( 'Shift Toward', 'revealer' ),
			'fade'          => esc_html__( 'Fade', 'revealer' ),
			'scale'         => esc_html__( 'Scale', 'revealer' ),
			'perspective'   => esc_html__( 'Perspective', 'revealer' )
		];

		/** Render Animation select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['animation'],
			esc_html__('Animation', 'revealer' ),
			esc_html__('The type of transition animation.', 'revealer' ),
			[
				'name' => 'mdp_revealer_design_settings[animation]',
				'id' => 'mdp_revealer_design_settings_animation'
			]
		);

	}

	/**
	 * Render Pop-up Size field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function size() {

		/** Render Pop-up Size slider. */
		UI::get_instance()->render_slider(
			Settings::get_instance()->options['size'],
			50,
			800,
			1,
			esc_html__( 'Pop-up Size', 'revealer' ),
			esc_html__( 'Set the popup size in px. Current value: ', 'revealer' ) . '<strong>' . Settings::get_instance()->options['size'] . '</strong>' . esc_html__( ' px.', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[size]',
				'id' => 'mdp_revealer_settings_size',
				'class' => 'mdc-slider-width'
			]
		);

	}

	/**
	 * Render Image position field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function image_position() {

		/** Prepare options. */
		$options = [
			'without-image' => esc_html__( 'Without Image', 'revealer' ),
			'top'           => esc_html__( 'Top', 'revealer' ),
			'bottom'        => esc_html__( 'Bottom', 'revealer' ),
			'left'          => esc_html__( 'Left', 'revealer' ),
			'right'         => esc_html__( 'Right', 'revealer' ),
			'top-bottom'    => esc_html__( 'Top or Bottom', 'revealer' ),
			'top-left'      => esc_html__( 'Top or Left', 'revealer' ),
			'top-right'     => esc_html__( 'Top or Right', 'revealer' ),
			'bottom-left'   => esc_html__( 'Bottom or Left', 'revealer' ),
			'bottom-right'  => esc_html__( 'Bottom or Right', 'revealer' ),
			'random'        => esc_html__( 'Random', 'revealer' ),
		];

		/** Render Image position select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['image_position'],
			esc_html__('Image position', 'revealer' ),
			esc_html__('Image position in tooltip window', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[image_position]',
				'id' => 'mdp_revealer_settings_image_position'
			]
		);

	}

	/**
	 * Render Image size field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function image_size() {
	    global $_wp_additional_image_sizes;

	    /** Get all available image sizes. */
		$image_sizes = [];
		$default_image_sizes = [ 'thumbnail', 'medium', 'large' ];
		foreach ( $default_image_sizes as $size ) {
			$image_sizes[$size] = [
				'width'  => intval( get_option( "{$size}_size_w" ) ),
				'height' => intval( get_option( "{$size}_size_h" ) ),
				'crop'   => get_option( "{$size}_crop" ) ? get_option( "{$size}_crop" ) : false,
			];
		}
		if ( isset( $_wp_additional_image_sizes ) && count( $_wp_additional_image_sizes ) ) {
			$image_sizes = array_merge( $image_sizes, $_wp_additional_image_sizes );
		}

		/** Prepare options. */
		$options = [];
		foreach ( $image_sizes as $k => $size ) {
			$options [$k] = $k;
        }

		/** Render Image size select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['image_size'],
			esc_html__('Image size', 'revealer' ),
			esc_html__('Image size from Media library presets', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[image_size]',
				'id' => 'mdp_revealer_settings_image_size'
			]
		);

	}

	/**
	 * Render Author position field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function author_position() {

		/** Prepare options. */
		$options = [
            'without-author'    => esc_html__( 'Hide author', 'revealer' ),
            'top'               => esc_html__( 'Top', 'revealer' ),
            'bottom'            => esc_html__( 'Bottom', 'revealer' )
		];

		/** Render Author position select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['author_position'],
			esc_html__('Author position', 'revealer' ),
			esc_html__('Author Name position inside the tooltip', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[author_position]',
				'id' => 'mdp_revealer_settings_author_position'
			]
		);

	}

	/**
	 * Render Categories position field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function category_position() {

		/** Prepare options. */
		$options = [
			'without-category' => esc_html__( 'Hide categories', 'revealer' ),
			'top' => esc_html__( 'Top', 'revealer' ),
			'bottom' => esc_html__( 'Bottom', 'revealer' )
		];

		/** Render Categories position select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['category_position'],
			esc_html__('Categories position', 'revealer' ),
			esc_html__('Categories position inside the tooltip', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[category_position]',
				'id' => 'mdp_revealer_settings_category_position'
			]
		);

	}

	/**
	 * Render Comments position field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function comments_position() {

		/** Prepare options. */
		$options = [
            'without-comments'  => esc_html__( 'Hide comments', 'revealer' ),
			'top'               => esc_html__( 'Top', 'revealer' ),
			'bottom'            => esc_html__( 'Bottom', 'revealer' )
		];

		/** Render Comments position select. */
		UI::get_instance()->render_select(
			$options,
			Settings::get_instance()->options['comments_position'],
			esc_html__('Comments position', 'revealer' ),
			esc_html__('Comments counter position inside the tooltip', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[comments_position]',
				'id' => 'mdp_revealer_settings_comments_position'
			]
		);

	}

	/**
	 * Render Excerpt Length field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function excerpt_length() {

		/** Render Excerpt Length slider. */
		UI::get_instance()->render_slider(
			Settings::get_instance()->options['excerpt_length'],
			1,
			150,
			1,
			esc_html__( 'Excerpt Length', 'revealer' ),
			esc_html__( 'The length of the text in popup in words. Current value: ', 'revealer' ) . '<strong>' . Settings::get_instance()->options['excerpt_length'] . '</strong>' . esc_html__( ' words.', 'revealer' ),
			[
				'name' => 'mdp_revealer_settings[excerpt_length]',
				'id' => 'mdp_revealer_settings_excerpt_length',
				'class' => 'mdc-slider-width'
			]
		);

	}

	/**
	 * Render Background Color field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function bgcolor() {

		/** Render Background Color colorpicker. */
		UI::get_instance()->render_colorpicker(
			Settings::get_instance()->options['bgcolor'],
			esc_html__( 'Background Color', 'revealer' ),
			esc_html__( 'Tooltip Background Color', 'revealer' ),
			[
				'name'      => 'mdp_revealer_design_settings[bgcolor]',
				'id'        => 'mdp_revealer_design_settings_bgcolor',
				'readonly'  => 'readonly'
			]
		);

	}

	/**
	 * Render Text Color field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public static function text_color() {

		/** Render Text Color colorpicker. */
		UI::get_instance()->render_colorpicker(
			Settings::get_instance()->options['text_color'],
			esc_html__( 'Text Color', 'revealer' ),
			esc_html__( 'Tooltip Text Color', 'revealer' ),
			[
				'name'      => 'mdp_revealer_design_settings[text_color]',
				'id'        => 'mdp_revealer_design_settings_text_color',
				'readonly'  => 'readonly'
			]
		);

	}

	/**
	 * Render CSS field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public static function custom_css() {
		?>
		<div>
            <label>
                <textarea
                    id="mdp_custom_css_fld"
                    name="mdp_revealer_css_settings[custom_css]"
                    class="mdp_custom_css_fld"><?php echo esc_textarea( Settings::get_instance()->options['custom_css'] ); ?></textarea>
            </label>
			<p class="description"><?php esc_html_e( 'Add custom CSS here.', 'revealer' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Render "SettingsFields Saved" nags.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 **/
	public static function render_nags() {

		/** Did we try to save settings? */
		if ( ! isset( $_GET['settings-updated'] ) ) { return; }

		/** Are the settings saved successfully? */
		if ( $_GET['settings-updated'] === 'true' ) {

			/** Render "SettingsFields Saved" message. */
			UI::get_instance()->render_snackbar( esc_html__( 'SettingsFields saved!', 'revealer' ) );
		}

		if ( ! isset( $_GET['tab'] ) ) { return; }

		if ( strcmp( $_GET['tab'], "activation" ) == 0 ) {

			if ( PluginActivation::get_instance()->is_activated() ) {

				/** Render "Activation success" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'revealer' ), 'success', 5500 );

			} else {

				/** Render "Activation failed" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'revealer' ), 'error', 5500 );

			}

		}

	}

	/**
	 * Main SettingsFields Instance.
	 *
	 * Insures that only one instance of SettingsFields exists in memory at any one time.
	 *
	 * @static
	 * @return SettingsFields
	 * @since 1.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof SettingsFields ) ) {
			self::$instance = new SettingsFields;
		}

		return self::$instance;
	}
	
} // End Class SettingsFields.
